clear

%**************************************************************************
%
%
%                           QCENT_PHASE.M (COPY OF QCENT.M, phase enabled)
%                                           Ioannis Karafyllidis  
%                                                              Start: 5-12-2003
%
% PURE-STATE ENTANGLEMENT COMPUTATION USING DENCITY MATRICES AND PARTIAL TRACES
%
%               STUDY OF ENTANGLEMENT SPREADING IN QUANTUM COMPUTATIONS
%
%                 ENTANGLEMEN SPREADING DURING Q.F.T.
%
%**************************************************************************

%***************************************************************
%            Quantum Register and Qbit Representation
%
% QR=[0 0 1 1] --> |0011> = |3>,  QR=[0 1 0 0] --> |0100> = |4>
%
%             Qbit numbering in the Quantum Register
%   |qbit n> -------
%   ......
%   |qbit 2> -------
%   |qbit 1> -------
%   |qbit 0> -------
%
%           Quantum registers for the two above examples
%    
%      1st                2nd
%         |0>                |0>
%         |0>                |1>
%         |1>                |0>
%         |1>                |0>
%
%***************************************************************
                                      
                                         
                                         
% The Function: ik_outplot2(QSTATE,t,nst)
% will be used to plot the output. This function puts 8 and 10 ticks 
% on the y and x axes, respectivelly


% QG Gate specification
% 0: Identity matrix (do nothing operation)
% 1: H gate, 2: PHG gate, 3: CNOT gate, 4: CCNOT gate, 5,8: F gate.
% 6: CPHG gate, 9: NO gate, Control Bit.
% elements in each time step.


% QG columns--> No. of Posible States
% QG rows ----> No. of Computation (Time) steps t
% t: No. of Computation (Time) steps t 

%"imag" will be used as the imaginary symbol "i" in order to avoid confusion

imag=0+1.0000i;
% In  ...3 9... == ... 9 3 ...


% NUMBER OF QBITS (lines) AND COMPUTATION STEPS
%        Read from File 


fo4 = fopen('QuantLinstp.txt','r');
QIN=fscanf(fo4,'%d',[2]);
stnetl=fclose(fo4);

     % n : Number of qbits
     % nst: Number of Possible states 
     n=QIN(1);
     nst=2^n;
     % ng(i) : Number of Gates at time step i
     ng=n;
     
     t=QIN(2); % Number of computation steps
%    ----------------------------------------

%             QUANTUM GATE CONFIGURATION
%               Read from File

fo2 = fopen('QuantGate.txt','r');
QG=fscanf(fo2,'%d',[t,ng]);
stnetl=fclose(fo2);


%             Phase Angles for Phase Gates
%             0: No phase or no phase gate, real: phase angle

fo3 = fopen('QuantPhase.txt','r');
PHA=fscanf(fo3,'%f',[t,ng]);
stnetl=fclose(fo3);

% 1. - - - -  INITIAL (INPUT) STATES - - - - -
%               Read from File

fo1 = fopen('QuantReg.txt','r');
QR=fscanf(fo1,'%d',[n]);
stnetl=fclose(fo1);


% - - - -  END OF INITIAL (INPUT) STATES - - - - -
 
 
 
% 2. - - - -  INPUT TENSOR PRODUCT - - - - - - -
 
%       Quantum Computer INput (QCIN) 
%is the Tensor Product of the input qbit matrices 
% QCIN is a column with 2.^n elements 
 
 QCIN=ik_qcin(n,QR);
 
 QOUT(:,1)=QCIN;
 QPR(:,1)=QCIN;

% - - - - END OF INPUT TENSOR PRODUCT - - - - - - -
 
 

% 3. - - - - -  EVOLUTION - - - - - - - - - - - - - 


% t : Number of Computation (time) steps

for i=1:t
   
% State column matrix
if i==1
   Qstate=QCIN;
else
   Qstate=Qrstp;
end
   
% Get Gate type for this computation step

QGSTEP=QG(i,:);

% Get phase angles for this computation step

PHASE=PHA(i,:);
% Gate tensor product

QGTP=ik_qg(ng,QGSTEP,PHASE);   
   
% Evolve one step (Multiply QGTP*Qrstp)

Qnew=QGTP*Qstate;

% Write to the output matrix QOUT
  %Write Probability Amplitude to be used in calculations
  QOUT(:,i+1)=Qnew;
  %Write Probability to be used for output graphics
%  QPR(:,i+1)=abs(Qnew).^2;
% Make the old matrix (Qrstp) new (Qnew)

Qrstp=Qnew;
   
end  % Computation Steps

% - - - -  END  EVOLUTION - - - - - - - - - - - - - 


% 4. - - - PROBABILITY CALCULATION - - - - - - - 
  %Write Probability to be used for output graphics
  
for ip=1:nst
    for jp=1:t+1
        QPR(ip,jp)=(abs(QOUT(ip,jp)))^2;
    end
end

%   PROBABILITY CHECK
%   Each column of the Probability Evolution 
%   Matrix QPR must add up to 1, in order to 
%   preserve probability during evolution.
for k=1:t+1
    Sum=0.0;
    for l=1:nst
       Sum=Sum+QPR(l,k);
    end  
    QEV_TST(k)=Sum;
end
Matrix_dummy=QEV_TST % This is to print QEV_TST into Workspace

% - - - - - END OF PROBABILITY CHECK - - - 


% Write on the workspace the final probability amplitudes QOUT(:,t)
% and the final probabilities QPR(:t)
%str1=(' - - - - PROBABILITY  AMPLITUDES - - - - - ')
%QOUT(:,t+1)
%str2=(' - - - - - - -PROBABILITIES - - - - - - - -')
%QPR(:,t+1)


% 5. - - - - OUTPUT GRAPHICS - - - - - - - - - - - 
figure('NumberTitle','off','Name','D.U.Th.  Quantum Computer Simulator                     Dr. Ioannis Karafyllidis');
ddd=ik_outplot(QPR,t,nst);

% - - - END OUTPUT GRAPHICS - - - - - - - - - - - 

% 6. - - - - PHASE PLOT - - - - - - - - - - - - - 
ppp=ik_phplot(QOUT,t,nst);

% - - - END PHASE  PLOT - - - - - - - - - - - - -

% 7. - - - ENTANGLEMENT CALCULATION AND PLOT - - - - - - - -

ENTANGLEMENT=ik_ent(n,t,QOUT);



  