function QGSTP=ik_qg(ng,QG,PHASE)

% INCLUDES CONTROL QBITS

%"imag" will be used as the imaginary symbol "i" in order to avoid confusion

imag=0+1.0000i;

% This function produces the tensor product of the 
% quantum gate matrices for a "time step".

% ng   : Number of gates in step 
% QC  : Gate matrix for this time step   [1   3   2   4   2  ...]
% Qph : The phase matrix of phase gates  [0   0  90   0   30 ...]

% The pahse matrix corresponds to QC matrix, is zero in all gates
% and in phase gates the corresponding element is the phase

% Angles are given id degrees and calculated in radians
%dotr: converts degrees to radians

dtor=((2*pi)/360.0000);

% A. - - - - - - Quantum Gate Matrices - - - - - -
% 0: Identity matrix (do nothing operation)
% 1: H gate, 2: PHG gate, 3: CNOT gate, 4: CCNOT gate, 5,8: F gate.
% 6: CPHG gate, 9: NO gate,Control qbit.
% elements in each time step.

H= (1/(sqrt(2)))*[1  1; 
                  1 -1];
    
    pha=0; % A starting value to define the matrix          
PHG= [ 1 0;
       0 (cos(pha)+imag*sin(pha))]; % pha= exp(ph*imag)=cos(ph)+isin(ph)  ph:phase angle
           
   
   
    phb=0; % A starting value to define the matrix    
CPHG=[ 1 0; 
       0 (cos(phb)+imag*sin(phb))]; % phb= exp(ph*imag)=cos(ph)+isin(ph)  ph:phase angle
    
 

CNOT=[0 1; 
      1 0];
   
   
CCNOT=[0 1;
       1 0];
    
    
    F=[1 0 0 0;
       0 0 1 0;
       0 1 0 0;
       0 0 0 1];
 

% - - - - - - End of Quantum Gate Matrices - - - - - - - - - -

% B. - - - Ordered Tensor Product of Gate Matrices - - - - - -


Cbn=0; % number of Control Qbits

%  Last gate 
switch QG(ng)
case 0
   GAA=eye(2);
case 1
   GAA=H;
case 2
   pha=PHASE(ng)*dtor;
   PHG= [ 1 0;
          0 (cos(pha)+imag*sin(pha))];
   GAA=PHG;  % Phase gate
case 3
   GAA=CNOT;
case 4
   GAA=CCNOT;
case 5
   GAA=F;
case 6
   phb=PHASE(ng)*dtor; 
   CPHG=[1 0; 
         0 (cos(phb)+imag*sin(phb))];
   GAA=CPHG; % Control-Phase gate
case 8
    GAA=1;% Fredkin cover gate
case 9
   GAA=eye(2); 
   Cbn=Cbn+1;
end
QGSTPNC=GAA;

% Tensor product

for j=1:ng-1
   jj=ng-j;
   if (j==1)
           switch QG(jj)
           case 0
              GAB=eye(2);
           case 1
              GAB=H;
           case 2
              pha=PHASE(jj)*dtor;
              PHG= [ 1 0;
                     0 (cos(pha)+imag*sin(pha))];
              GAB=PHG;  % Phase gate
           case 3
              GAB=CNOT;
           case 4
              GAB=CCNOT;
           case 5
              GAB=F;
           case 6
               phb=PHASE(jj)*dtor; 
               CPHG=[1 0; 
                     0 (cos(phb)+imag*sin(phb))];
               GAB=CPHG; % Control-Phase gate
           case 8
               GAB=1; %Fredkin cover gate
           case 9
              GAB=eye(2); 
              Cbn=Cbn+1;   
           end
           
   QGSTPNC=kron(GAB,GAA);
   
   else
           switch QG(jj)
           case 0
              GAB=eye(2);
           case 1
              GAB=H;
           case 2
              pha=PHASE(jj)*dtor;
              PHG= [ 1 0;
                     0 (cos(pha)+imag*sin(pha))];
              GAB=PHG;  % Phase gate
           case 3
              GAB=CNOT;
           case 4
              GAB=CCNOT;
           case 5
              GAB=F;
           case 6
              phb=PHASE(jj)*dtor; 
              CPHG=[1 0; 
                    0 (cos(phb)+imag*sin(phb))];
              GAB=CPHG; % Control-Phase gate
           case 8
           GAB=1; %Fredkin cover gate   
           case 9
              GAB=eye(2); 
              Cbn=Cbn+1;
           end  
           
   QGSTPNC=kron(GAB,QGSTPNC);        
    
    
   end   % end if
    
end   % end for      
       
% - - - End of Ordered Tensor Product of Gate Matrices - - - - 


% C. - - - Control qbits - - - - - -

for k=1:ng
    if (QG(k)==9);
       QGR(k)=1;
   else
       QGR(k)=0;
   end
end

QCB=ik_cntrqbit(ng,QGR);

nr=2^ng;
if (Cbn==0)
    QGSTP=QGSTPNC;
    
    
elseif (Cbn==1)
    for ir=1:nr
        if (QCB(ir)==1)
            QGSTPNC(ir,:)=0;
            QGSTPNC(ir,ir)=1;
        end
    end
QGSTP=QGSTPNC;  



elseif (Cbn==2)
 
     for ir=1:nr
        if (QCB(ir)<101)
            QGSTPNC(ir,:)=0;
            QGSTPNC(ir,ir)=1;
        end
     end   
QGSTP=QGSTPNC;
    
elseif  (Cbn>=3)
    'More than two control qbits were used.'
     QGSTP=QGSTPNC;
end
    
 
 QCB;

% - - - - End of Control qbits - - - - - -